#include "tester_func.h"

TESTER_TestHandleTypeDef hTestDiode;

/**s
  * @brief    Инициализация хендла тестера диодов
  */
void TESTER_HandleInit(TESTER_TestHandleTypeDef *htest, TESTER_LEDsTypeDef *leds)
{
  /* Настройка структуры АЦП */
  htest->adc = &tester_adc;  
  htest->leds = leds;
  TESTER_ADC_StructInit(htest->adc);
    
  htest->continuous_buff_size = 20;
}


/**
  * @brief    Тест диодов: подключение прямого напряжения
  */
void TESTER_TestDiode_Forward(TESTER_TestHandleTypeDef *htest)
{
  /* Задержка, перед началом работы */
  TESTER_Delay(&htest->SwTimings.ticks_before_test);
  TESTER_LED_TestingDiodeForward(&htest->leds->LED1);
  
  /* Включение континиус АЦП */
  HAL_ADC_Start(htest->adc->hadc);  
  
  /* Прямое включение диода */
  TESTER_Connect_Phase(&htest->SwPhaseForward);
  
  /* Прямое включение на определенное время */
  uint32_t tickstart = HAL_GetTick();
  HAL_StatusTypeDef res;
  while(1)
  {
    /* Считывание АЦП пока таймаут истечет */
    res = ADC_ReadContinuous(htest->adc, htest->continuous_buff_size, tickstart, htest->SwTimings.msticks_for_forward);
    
    if(res != HAL_OK)
      break;
  }
  
  /* Отключение питания от диода */
  TESTER_Disconnect_Phase(&htest->SwPhaseForward);
  TESTER_Disconnect_Phase(&htest->SwPhaseReverse);
  
  /* Выключение континиус АЦП */
  HAL_ADC_Stop(htest->adc->hadc);
  
  TESTER_LED_WaitForAction(&htest->leds->LED1);  
  
  htest->DiodeForwardVolt = htest->adc->chAdc.U_Current;
}

/**
  * @brief    Тест диодов: подключение обратного напряжения
  */
void TESTER_TestDiode_Reverse(TESTER_TestHandleTypeDef *htest)
{
  /* Задержка, перед началом работы */
  TESTER_Delay(&htest->SwTimings.ticks_before_test);
  TESTER_LED_TestingDiodeReverse(&htest->leds->LED1);
  
  /* Включение АЦП */
  ADC_DMA_StartRead(htest->adc);
  
  /* Задержка, перед предполагаемым скачком */
  TESTER_Delay(&htest->SwTimings.ticks_before_expected_peak);
  
  /* Обратное включение диода */
  TESTER_Connect_Phase(&htest->SwPhaseReverse);
  
  /* Обратное включение на определенное время */
  TESTER_Delay(&htest->SwTimings.ticks_before_disconnect);
  
  /* Отключение питания от диода */
  TESTER_Disconnect_Phase(&htest->SwPhaseReverse);
  TESTER_Disconnect_Phase(&htest->SwPhaseForward);
  
  /* Обработка DMA */
  ADC_DMA_ReadForPeak(htest->adc, ADC_READ_TIMEOUT_MS);
  
  TESTER_LED_WaitForAction(&htest->leds->LED1);
  
  htest->DiodePeakVolt = htest->adc->chAdc.U_Current;  
}


/**
  * @brief    Тест диодов: подключение прямого, а потом обратного напряжения
  */
void TESTER_TestDiode_SwitchConnection(TESTER_TestHandleTypeDef *htest)
{ 
  /* Задержка, перед началом работы */
  TESTER_Delay(&htest->SwTimings.ticks_before_test);
  TESTER_LED_TestingDiodeForward(&htest->leds->LED1);
  
  /* Прямое включение диода */
  TESTER_Connect_Phase(&htest->SwPhaseForward);
  
  /* Прямое включение на определенное время */
  msDelay(htest->SwTimings.msticks_for_forward);
  htest->DiodeForwardVolt = htest->adc->chAdc.U_Current;
  TESTER_LED_TestingDiodeReverse(&htest->leds->LED1);
  
  /* Включение АЦП */
  ADC_DMA_StartRead(htest->adc);
  
  /* Ожидается задержка, перед предполагаемым скачком */
  TESTER_Delay(&htest->SwTimings.ticks_before_expected_peak);
  
  /* Переход из прямого включения в обратное */
  TESTER_Reconnect_TwoPhases(&htest->SwPhaseForward, &htest->SwPhaseReverse, &htest->SwTimings.ticks_deadtime);
  
  /* Обратное включение на определенное время */
  TESTER_Delay(&htest->SwTimings.ticks_before_disconnect);
  
  /* Отключение питания от диода */
  TESTER_Disconnect_Phase(&htest->SwPhaseReverse);
  TESTER_Disconnect_Phase(&htest->SwPhaseForward);
  
  /* Обработка DMA */
  ADC_DMA_ReadForPeak(htest->adc, ADC_READ_TIMEOUT_MS);
  
  TESTER_LED_WaitForAction(&htest->leds->LED1);
  
  htest->DiodePeakVolt = htest->adc->chAdc.U_Current;  
}

/**
  * @brief    Подключить фазу с помощью ключей (пинов) в TESTER_PhaseSwitchTypeDef
  */
void TESTER_Connect_Phase(TESTER_PhaseSwitchTypeDef *DCSw)
{
#ifdef USE_HAL_GPIO_FUNCTIONS // in tester_config.h
  HAL_GPIO_WritePin(DCSw->SW_Port, DCSw->SwHI_Pin | DCSw->SwLO_Pin, PHASE_CONNECT);
#else // USE_HAL_GPIO_FUNCTIONS  
  #if (PHASE_CONNECT == 1)
    DCSw->SW_Port->BSRR = DCSw->SwHI_Pin | DCSw->SwLO_Pin;
  #else //PHASE_CONNECT == 1
    DCSw->SW_Port->BSRR = (DCSw->SwHI_Pin | DCSw->SwLO_Pin) << 16;
  #endif //PHASE_CONNECT == 1
#endif //USE_HAL_GPIO_FUNCTIONS
}
/**
  * @brief    Отключить фазу с помощью ключей (пинов) в TESTER_PhaseSwitchTypeDef
  */
void TESTER_Disconnect_Phase(TESTER_PhaseSwitchTypeDef *DCSw)
{
#ifdef USE_HAL_GPIO_FUNCTIONS
  HAL_GPIO_WritePin(DCSw->SW_Port, DCSw->SwHI_Pin | DCSw->SwLO_Pin, PHASE_DISCONNECT);
#else // USE_HAL_GPIO_FUNCTIONS
  #if (PHASE_DISCONNECT == 0)
    DCSw->SW_Port->BSRR = (DCSw->SwHI_Pin | DCSw->SwLO_Pin) << 16;
  #else //PHASE_CONNECT == 1
    DCSw->SW_Port->BSRR = DCSw->SwHI_Pin | DCSw->SwLO_Pin;
  #endif //PHASE_CONNECT == 1
#endif //USE_HAL_GPIO_FUNCTIONS
}

/**
  * @brief    Переключить две фазы с помощью ключей (пинов) в TESTER_PhaseSwitchTypeDef
  */
void TESTER_Reconnect_TwoPhases(TESTER_PhaseSwitchTypeDef *SwPhaseA, TESTER_PhaseSwitchTypeDef *SwPhaseB, TESTER_TicksDelayTypeDef *deadtime)
{
#ifdef USE_HAL_GPIO_FUNCTIONS
  HAL_GPIO_WritePin(SwPhaseA->SW_Port, SwPhaseA->SwHI_Pin | SwPhaseA->SwLO_Pin, PHASE_DISCONNECT);
  
  #ifndef RECONNECT_WITHOUT_DEADTIME
  /* Ожидается задержка дедтайм */
  TESTER_Delay(deadtime);
  #endif //RECONNECT_WITHOUT_DEADTIME
  
  HAL_GPIO_WritePin(SwPhaseB->SW_Port, SwPhaseB->SwHI_Pin | SwPhaseB->SwLO_Pin, PHASE_CONNECT);
#else // USE_HAL_GPIO_FUNCTIONS
  #ifdef RECONNECT_WITHOUT_DEADTIME
    #if (PHASE_CONNECT == 1) && (PHASE_DISCONNECT == 0)
      SwPhaseA->SW_Port->BSRR = ((SwPhaseB->SwHI_Pin | SwPhaseB->SwLO_Pin) | ((SwPhaseA->SwHI_Pin | SwPhaseA->SwLO_Pin) << 16));
    #elif (PHASE_CONNECT == 1) && (PHASE_DISCONNECT == 0)
      SwPhaseA->SW_Port->BSRR = (((SwPhaseB->SwHI_Pin | SwPhaseB->SwLO_Pin) << 16)| (SwPhaseA->SwHI_Pin | SwPhaseA->SwLO_Pin));
    #endif //PHASE_CONNECT && PHASE_DISCONNECT
  #else //RECONNECT_WITHOUT_DEADTIME  
    #if (PHASE_DISCONNECT == 0)
      SwPhaseA->SW_Port->BSRR = (SwPhaseA->SwHI_Pin | SwPhaseA->SwLO_Pin) << 16;
    #else //PHASE_DISCONNECT == 1
      SwPhaseA->SW_Port->BSRR = SwPhaseA->SwHI_Pin | SwPhaseA->SwLO_Pin;
    #endif //PHASE_DISCONNECT
  
    /* Ожидается задержка дедтайм */
    TESTER_Delay(deadtime);
  
    #if (PHASE_CONNECT == 1)
      SwPhaseB->SW_Port->BSRR = SwPhaseB->SwHI_Pin | SwPhaseB->SwLO_Pin;
    #else //PHASE_CONNECT == 0
      SwPhaseB->SW_Port->BSRR = (SwPhaseB->SwHI_Pin | SwPhaseB->SwLO_Pin) << 16;
    #endif //PHASE_CONNECT
  #endif //RECONNECT_WITHOUT_DEADTIME
#endif //USE_HAL_GPIO_FUNCTIONS
}

/**
  * @brief    Формирование задержки (в тиках или миллисекундная)
  */
void TESTER_Delay(TESTER_TicksDelayTypeDef *tickdelay)
{
  /* если миллисекундная задержка выключена */
  if(tickdelay->msdelay == 0)
  {
    /* Задержка, в тиках  */
    for(int i = 0; i < tickdelay->ticks; i++);
  }
  /* если миллисекундная задержка включена */
  else
  {
    /* Задержка, в миллисекундах */
    msDelay(tickdelay->ticks);
  }
}