/**
**************************************************************************
* @file tester_func.h
* @brief Заголовочный файл для функций МЗКТ.
**************************************************************************
@details

*************************************************************************/

	
#ifndef _TESTER_FUNC_H_
#define _TESTER_FUNC_H_

#include "tester_adc_func.h"
#include "tester_interface_func.h"


/**
  * @brief Структура для пинов, которые отвечают за ключи, которые подключают фазы
  */
typedef struct
{
  GPIO_TypeDef *SW_Port;      ///< Порт первого ключа для фазы
  uint32_t      SwHI_Pin;    ///< Пин первого ключа для фазы
  uint32_t      SwLO_Pin;    ///< Пин второго ключа для фазы
}TESTER_PhaseSwitchTypeDef;


/**
  * @brief Структура для тайминга с флагом миллисекундой или тиковой (пустой for) задержкой
  */
typedef struct
{
  unsigned  msdelay:1;
  uint32_t  ticks;
}TESTER_TicksDelayTypeDef;

/**
  * @brief Структура с таймингами для тестера 
  */
typedef struct
{
  uint32_t                  msticks_for_forward;        ///< миллисекундная задержка для положительного напряжения @ref TESTER_TestDiode_Forward
  TESTER_TicksDelayTypeDef  ticks_before_test;          ///< задержка перед началом тестирования (мс или тики for())
  TESTER_TicksDelayTypeDef  ticks_deadtime;             ///< задержка между включением АЦП и предполагаемым скачком напряжения, чтобы точно поймать его в буфере АЦП (мс или тики for())
  TESTER_TicksDelayTypeDef  ticks_before_expected_peak; ///< задержка между включением АЦП и предполагаемым скачком напряжения, чтобы точно поймать его в буфере АЦП (мс или тики for())
  TESTER_TicksDelayTypeDef  ticks_before_disconnect;    ///< задержка перед выключением фаз (мс или тики for())
}TESTER_SwitchTimingsTypeDef;

/**
  * @brief Структура-дескриптор для управления тестированием диодов
  */
typedef struct
{
  TESTER_ADCTypeDef *adc;                 ///< указатель на структуру АЦП тестера
  
  TESTER_SwitchTimingsTypeDef SwTimings;  ///< Тайминги для ключей
  
  TESTER_PhaseSwitchTypeDef SwPhaseForward;      ///< Пины ключей фаз для прямого включения
  TESTER_PhaseSwitchTypeDef SwPhaseReverse;      ///< Пины ключей фаз для обратного включения
  
  uint32_t          continuous_buff_size;
  
  float             DiodeForwardVolt;
  float             DiodePeakVolt;
  
  TESTER_LEDsTypeDef *leds;
}TESTER_TestHandleTypeDef;
extern TESTER_TestHandleTypeDef hTestDiode;



/* Инициализация хендла тестера диодов */
void TESTER_HandleInit(TESTER_TestHandleTypeDef *htest, TESTER_LEDsTypeDef *leds);

/* Тест диодов: подключение прямого напряжения */
void TESTER_TestDiode_Forward(TESTER_TestHandleTypeDef *htest);
/* Тест диодов: подключение обратного напряжения */
void TESTER_TestDiode_Reverse(TESTER_TestHandleTypeDef *htest);
/* Тест диодов: подключение прямого, а потом обратного напряжения */
void TESTER_TestDiode_SwitchConnection(TESTER_TestHandleTypeDef *htest);

/* Подключить фазу с помощью ключей (пинов) в TESTER_PhaseSwitchTypeDef */
void TESTER_Connect_Phase(TESTER_PhaseSwitchTypeDef *DCSw);
/* Отключить фазу с помощью ключей (пинов) в TESTER_PhaseSwitchTypeDef */
void TESTER_Disconnect_Phase(TESTER_PhaseSwitchTypeDef *DCSw);
/* Переключить две фазы с помощью ключей (пинов) в TESTER_PhaseSwitchTypeDef */
void TESTER_Reconnect_TwoPhases(TESTER_PhaseSwitchTypeDef *SwPhaseA, TESTER_PhaseSwitchTypeDef *SwPhaseB, TESTER_TicksDelayTypeDef *deadtime);
/* Формирование задержки (в тиках или миллисекундная) */
void TESTER_Delay(TESTER_TicksDelayTypeDef *tickdelay);
#endif //_TESTER_FUNC_H_

