/**
**************************************************************************
* @file tester_func.h
* @brief Заголовочный файл для функций МЗКТ.
**************************************************************************
@details

*************************************************************************/

	
#ifndef _TESTER_MAIN_H_
#define _TESTER_MAIN_H_

#include "tester_func.h"


/**
  * @brief Структура для вызова различных функций
	*/
typedef struct
{
  unsigned test_diode_reverse:1;
  unsigned test_diode_switch_connection:1;
  unsigned test_diode_forward:1;
  
  unsigned disable_reset_call:1;
}function_calls;

/**
  * @brief Структура для вызова различных функций
	*/
typedef struct
{
  unsigned flag_init_done:1;
  unsigned flag_test_active:1;
}TESTER_FlagsTypeDef;

typedef enum
{
  TEST_DISABLE    = 0,
  TEST_FORWARD   = 1,
  TEST_REVERSE   = 2,
  TEST_SWITCH_CONNECTION     = 3
}TESTER_TestModeTypeDef;


/**
  * @brief Структура проекта тестер
	*/
typedef struct
{
  function_calls              func;     ///< вызов разных функций
  
  TESTER_TestModeTypeDef      mode;     ///< режим тестера
  
  TESTER_TestHandleTypeDef    *htest;   ///< дескриптор тестера
  
  RS_HandleTypeDef            *hmodbus;
  MB_DataStructureTypeDef     *mbdata;
  
  
  TESTER_SwitchStartTypeDef   SwStart;  ///< структура кнопки старта
  TESTER_LEDsTypeDef          leds;     ///< структура светодиодов
  
  
  TESTER_FlagsTypeDef         f;
  
  uint32_t        delay;        ///< задержка главного цикла
  uint32_t        delay_en:1;   ///< включить задержку в главном цикле
}TESTER_ProjectTypeDef;
extern TESTER_ProjectTypeDef TESTER;



/* Инициализация всех модулей для работы тестера */
void TESTER_Init(TESTER_ProjectTypeDef *tester);
/* Функция подготовки тестера к while(1) */
void TESTER_pre_while(TESTER_ProjectTypeDef *tester);
/* Главная функция работы тестера */
void TESTER_main_while(TESTER_ProjectTypeDef *tester);
/*  Функция работы интерфейса */
void TESTER_InterfaceHandle(TESTER_ProjectTypeDef *tester);
/* Обновление настроек тестера */
void TESTER_UpdateSettings(TESTER_TestHandleTypeDef *htest, MB_DataStructureTypeDef *mbdata);
/* Инициализация тестера по дефолтным настрйокам в tester_config.h */
void TESTER_Set_Default_Settings(TESTER_ProjectTypeDef *tester);
/*Задеркжка главного цикла */
void TESTER_main_delay(TESTER_ProjectTypeDef *tester);
#endif //_TESTER_MAIN_H_

